@@ -409,6 +409,18 @@ typedef struct  _SFLExtended_TCP_info {
 
 #define  XDRSIZ_SFLEXTENDED_TCP_INFO 48
 
+  /* Physical or virtual host description
+     opaque = flow_data; enterprise = 0; format = 2210
+     Set Data source to all zeroes if unknown  */
+typedef struct _SFLExtended_entities {
+  uint32_t src_dsClass; /* Data Source associated with packet source */
+  uint32_t src_dsIndex;
+  uint32_t dst_dsClass; /* Data Source associated with packet destination */
+  uint32_t dst_dsIndex;
+} SFLExtended_entities;
+
+#define XDRSIZ_SFLEXTENDED_ENTITIES 16
+
 /* Extended socket information,
    Must be filled in for all application transactions associated with a network socket
    Omit if transaction associated with non-network IPC  */
@@ -590,6 +602,64 @@ typedef struct _SFLExtended_decap {
     uint32_t innerHeaderOffset;
 } SFLExtended_decap;
 
+/* Selected egress queue */
+/* Output port number must be provided in enclosing structure */
+/* opaque = flow_data; enterprise = 0; format = 1036 */
+typedef struct {
+  unsigned int queue;  /* eqress queue number selected for sampled packet */
+} SFLExtended_egress_queue;
+#define XDRSIZ_SFLEXTENDED_EGRESS_Q 4
+
+/* Software function */
+/* Name of software function generating this event */
+/* opaque = flow_data; enterprise = 0; format = 1038 */
+typedef struct _SFLExtended_function {
+  SFLString symbol;
+} SFLExtended_function;
+#define SFL_MAX_FUNCTION_SYMBOL_LEN 64
+
+/* Delay for sampled packet traversing switch */
+/* opaque = flow_data; enterprise = 0; format = 1039 */
+typedef struct {
+  unsigned int delay; /* transit delay in nanoseconds
+			 0xffffffff indicates value >= 0xffffffff */
+} SFLExtended_transit_delay;
+#define XDRSIZ_SFLEXTENDED_TRANSIT 4
+
+/* Queue depth for sampled packet traversing switch */
+/* extended_egress_queue structure must be included */
+/* opaque = flow_data; enterprise = 0; format = 1040 */
+typedef struct {
+  unsigned int depth;   /* queue depth in bytes */
+} SFLExtended_queue_depth;
+#define XDRSIZ_SFLEXTENDED_Q_DEPTH 4
+
+  // Devlink Trap Name
+  // opaque = flow_data; enterprise = 0; format = 1041
+  // https://www.kernel.org/doc/html/latest/networking/devlink/devlink-trap.html
+  // XDR spec:
+  //  struct extended_hw_trap {
+  //    string group<>; /* NET_DM_ATTR_HW_TRAP_GROUP_NAME */
+  //    string trap<>; /* NET_DM_ATTR_HW_TRAP_NAME */
+  //  }
+typedef struct _SFLExtended_hw_trap {
+  SFLString group;
+  SFLString trap;
+} SFLExtended_hw_trap;
+#define SFL_MAX_HW_TRAP_LEN 64
+
+  // Linux drop_monitor reason
+  // opaque = flow_data; enterprise = 0; format = 1042
+  // https://github.com/torvalds/linux/blob/master/include/net/dropreason.h
+  // XDR spec:
+  //  struct extended_linux_drop_reason {
+  //    string reason<>; /* NET_DM_ATTR_REASON */
+  //  }
+typedef struct _SFLExtended_linux_reason {
+  SFLString reason;
+} SFLExtended_linux_reason;
+#define SFL_MAX_LINUX_REASON_LEN 64
+
 enum SFLFlow_type_tag {
   /* enterprise = 0, format = ... */
   SFLFLOW_HEADER    = 1,      /* Packet headers are sampled */
@@ -623,6 +693,12 @@ enum SFLFlow_type_tag {
     SFLFLOW_EX_DECAP_IN        = 1028,
     SFLFLOW_EX_VNI_OUT         = 1029,
     SFLFLOW_EX_VNI_IN          = 1030,
+  SFLFLOW_EX_EGRESS_Q          = 1036,
+  SFLFLOW_EX_FUNCTION          = 1038,
+  SFLFLOW_EX_TRANSIT           = 1039,
+  SFLFLOW_EX_Q_DEPTH           = 1040,
+  SFLFLOW_EX_HW_TRAP           = 1041,
+  SFLFLOW_EX_LINUX_REASON      = 1042,
   SFLFLOW_EX_SOCKET4       = 2100,
   SFLFLOW_EX_SOCKET6       = 2101,
   SFLFLOW_EX_PROXYSOCKET4  = 2102,
@@ -635,6 +711,7 @@ enum SFLFlow_type_tag {
   SFLFLOW_APP_ACTOR_TGT    = 2205, /* target */
   SFLFLOW_HTTP2            = 2206,
   SFLFLOW_EX_TCP_INFO      = 2209,
+  SFLFLOW_EX_ENTITIES      = 2210,
 };
 
 typedef union _SFLFlow_type {
@@ -668,6 +745,12 @@ typedef union _SFLFlow_type {
   SFLExtended_socket_ipv6 socket6;
   SFLExtended_vni tunnel_vni;
   SFLExtended_decap tunnel_decap;
+  SFLExtended_TCP_info tcp_info;
+  SFLExtended_entities entities;
+  SFLExtended_function function;
+  SFLExtended_egress_queue egress_queue;
+  SFLExtended_queue_depth queue_depth;
+  SFLExtended_transit_delay transit_delay;
 } SFLFlow_type;
 
 typedef struct _SFLFlow_sample_element {
@@ -682,6 +765,7 @@ enum SFL_sample_tag {
   SFLCOUNTERS_SAMPLE = 2,          /* enterprise = 0 : format = 2 */
   SFLFLOW_SAMPLE_EXPANDED = 3,     /* enterprise = 0 : format = 3 */
   SFLCOUNTERS_SAMPLE_EXPANDED = 4, /* enterprise = 0 : format = 4 */
+  SFLEVENT_DISCARDED_PACKET = 5,   /* enterprise = 0 : format = 5 */
   SFLRTMETRIC = ((4300 << 12) + 1002),
   SFLRTFLOW = ((4300 << 12) + 1003)
 };
@@ -1612,6 +1696,24 @@ typedef struct _SFLCounters_sample_expanded {
   SFLCounters_sample_element *elements;
 } SFLCounters_sample_expanded;
 
+#define SFL_DROP(name, code) SFLDrop_ ## name=code,
+typedef enum {
+#include "sflow_drop.h"
+} EnumSFLDropReason;
+#undef SFL_DROP
+
+typedef struct _SFLEvent_discarded_packet {
+  uint32_t sequence_number;
+  uint32_t ds_class; /* EXPANDED */
+  uint32_t ds_index; /* EXPANDED */
+  uint32_t drops;
+  uint32_t input; /* ifIndex */
+  uint32_t output; /* ifIndex */
+  EnumSFLDropReason reason;
+  uint32_t num_elements;
+  SFLFlow_sample_element *elements;
+} SFLEvent_discarded_packet;
+
 #define SFLADD_ELEMENT(_sm, _el) do { (_el)->nxt = (_sm)->elements; (_sm)->elements = (_el); } while(0)
 
 /* Format of a sample datagram */
